@@ -4,7 +4,7 @@ use rusty_v8 as v8;
 fn single_threaded_default_platform() {
   v8::V8::set_flags_from_string("--single_threaded");
   v8::V8::initialize_platform(
-    v8::new_single_threaded_default_platform().unwrap(),
+    v8::new_single_threaded_default_platform(false).make_shared(),
   );
   v8::V8::initialize();
 
